@@ -1,6 +1,4 @@
-
 using System;
-using System.Collections;
 using System.Collections.Generic;
 using System.Runtime.InteropServices;
 using System.Text;
@@ -19,6 +17,7 @@ namespace Tomboy
 		Gtk.ToolButton link_button;
 		NoteTextMenu text_menu;
 		Gtk.Menu plugin_menu;
+		Gtk.ImageMenuItem sync_menu_item;
 		Gtk.TextView editor;
 		Gtk.ScrolledWindow editor_window;
 		NoteFindBar find_bar;
@@ -160,13 +159,8 @@ namespace Tomboy
 				                            Gdk.ModifierType.ControlMask,
 				                            Gtk.AccelFlags.Visible);
 
-			// Have Esc key close the note window
-			if ((bool) Preferences.Get (Preferences.ENABLE_CLOSE_NOTE_ON_ESCAPE))
-				KeyPressEvent += KeyPressed;
-
-			// Watch the escape setting in GConf
-			Preferences.Client.AddNotify (Preferences.ENABLE_CLOSE_NOTE_ON_ESCAPE,
-					OnEscapeSettingChanged);
+			// Have Esc key close the find bar or note window
+			KeyPressEvent += KeyPressed;
 
 			// Increase Indent
 			global_keys.AddAccelerator (new EventHandler (ChangeDepthRightHandler),
@@ -185,6 +179,8 @@ namespace Tomboy
 
 		protected override bool OnDeleteEvent (Gdk.Event evnt)
 		{
+			Preferences.SettingChanged -= Preferences_SettingChanged;
+
 			CloseWindowHandler (null, null);
 			return true;
 		}
@@ -201,16 +197,6 @@ namespace Tomboy
 			Move (x, y);
 		}
 
-		void OnEscapeSettingChanged (object sender, NotifyEventArgs args)
-		{
-			// enable escape key
-			if ((bool) args.Value)
-				KeyPressEvent += KeyPressed;
-			// disable escape key
-			else
-				KeyPressEvent -= KeyPressed;
-		}
-
 		void KeyPressed (object sender, Gtk.KeyPressEventArgs args)
 		{
 			args.RetVal = true;
@@ -218,7 +204,10 @@ namespace Tomboy
 			switch (args.Event.Key)
 			{
 			case Gdk.Key.Escape:
-				CloseWindowHandler (null, null);
+				if (find_bar != null && find_bar.Visible)
+					find_bar.Hide ();
+				else if ((bool) Preferences.Get(Preferences.ENABLE_CLOSE_NOTE_ON_ESCAPE))
+					CloseWindowHandler (null, null);
 				break;
 			default:
 				args.RetVal = false;
@@ -508,17 +497,35 @@ namespace Tomboy
 
 			tb.Insert (new Gtk.SeparatorToolItem (), -1);
 
-			Gtk.ImageMenuItem item =
-			        new Gtk.ImageMenuItem (Catalog.GetString ("Synchronize Notes"));
-			item.Image = new Gtk.Image (Gtk.Stock.Convert, Gtk.IconSize.Menu);
-			item.Activated += SyncItemSelected;
-			item.Show ();
-			PluginMenu.Add (item);
+			sync_menu_item = new Gtk.ImageMenuItem (Catalog.GetString ("Synchronize Notes"));
+			sync_menu_item.Image = new Gtk.Image (Gtk.Stock.Convert, Gtk.IconSize.Menu);
+			sync_menu_item.Activated += SyncItemSelected;
+			sync_menu_item.Show ();
+			PluginMenu.Add (sync_menu_item);
+
+			// We might want to know when various settings are altered.
+			Preferences.SettingChanged += Preferences_SettingChanged;
+
+			// Update items based on configuration.
+			UpdateMenuItems ();
 
 			tb.ShowAll ();
 			return tb;
 		}
 
+		void Preferences_SettingChanged (object sender, EventArgs args)
+		{
+			// Update items based on configuration.
+			UpdateMenuItems ();
+		}
+
+		void UpdateMenuItems ()
+		{
+			// Is synchronization configured and active?
+			string sync_addin_id = Preferences.Get (Preferences.SYNC_SELECTED_SERVICE_ADDIN) as string;
+			sync_menu_item.Sensitive = !string.IsNullOrEmpty (sync_addin_id);
+		}
+
 		void SyncItemSelected (object sender, EventArgs args)
 		{
 			Tomboy.ActionManager ["NoteSynchronizationAction"].Activate ();
@@ -723,7 +730,7 @@ namespace Tomboy
 		Gtk.Button next_button;
 		Gtk.Button prev_button;
 
-		ArrayList current_matches;
+		List<Match> current_matches;
 		string prev_search_text;
 
 		InterruptableTimeout entry_changed_timeout;
@@ -754,10 +761,10 @@ namespace Tomboy
 			entry.Changed += OnFindEntryChanged;
 			entry.Activated += OnFindEntryActivated;
 			entry.Show ();
-			PackStart (entry, false, false, 0);
+			PackStart (entry, true, true, 0);
 
 			prev_button = new Gtk.Button (Catalog.GetString ("_Previous"));
-			prev_button.Image = new Gtk.Image (Gtk.Stock.GoBack, Gtk.IconSize.Menu);
+			prev_button.Image = new Gtk.Arrow (Gtk.ArrowType.Left, Gtk.ShadowType.None);
 			prev_button.Relief = Gtk.ReliefStyle.None;
 			prev_button.Sensitive = false;
 			prev_button.FocusOnClick = false;
@@ -765,8 +772,8 @@ namespace Tomboy
 			prev_button.Show ();
 			PackStart (prev_button, false, false, 0);
 
-			next_button = new Gtk.Button (Catalog.GetString ("Find _Next"));
-			next_button.Image = new Gtk.Image (Gtk.Stock.GoForward, Gtk.IconSize.Menu);
+			next_button = new Gtk.Button (Catalog.GetString ("_Next"));
+			next_button.Image = new Gtk.Arrow (Gtk.ArrowType.Right, Gtk.ShadowType.None);
 			next_button.Relief = Gtk.ReliefStyle.None;
 			next_button.Sensitive = false;
 			next_button.FocusOnClick = false;
@@ -924,7 +931,7 @@ namespace Tomboy
 
 			text = text.ToLower ();
 
-			string [] words = text.Split (' ', '\t', '\n');
+			string [] words = Search.SplitWatchingQuotes (text);
 
 			current_matches =
 			        FindMatchesInBuffer (note.Buffer, words);
@@ -1106,9 +1113,9 @@ namespace Tomboy
 			UpdateSensitivity ();
 		}
 
-		ArrayList FindMatchesInBuffer (NoteBuffer buffer, string [] words)
+		List<Match> FindMatchesInBuffer (NoteBuffer buffer, string [] words)
 		{
-			ArrayList matches = new ArrayList ();
+			List<Match> matches = new List<Match> ();
 
 			string note_text = buffer.GetSlice (buffer.StartIter,
 			                                    buffer.EndIter,
@@ -1184,6 +1191,14 @@ namespace Tomboy
 		Gtk.CheckMenuItem bullets;
 		Gtk.ImageMenuItem increase_indent;
 		Gtk.ImageMenuItem decrease_indent;
+		Gtk.MenuItem increase_font;
+		Gtk.MenuItem decrease_font;
+
+		// There is a bug in GTK+ that lends to improperly themed Menus
+		// and MenuItems when you derive from one of those classes; this
+		// is an internal Menu that sits around to provide proper theme
+		// information to override on this derived Menu
+		Gtk.Menu theme_hack_menu;
 
 		// Active when the text size is indeterminable, such as when in
 		// the note's title line.
@@ -1324,16 +1339,42 @@ namespace Tomboy
 			hidden_no_size = new Gtk.RadioMenuItem (small.Group, string.Empty);
 			hidden_no_size.Hide ();
 
+			increase_font = new Gtk.MenuItem (Catalog.GetString ("Increase Font Size"));
+			increase_font.AddAccelerator ("activate",
+						accel_group,
+						(uint) Gdk.Key.plus,
+						Gdk.ModifierType.ControlMask,
+						Gtk.AccelFlags.Visible);
+			increase_font.Activated += IncreaseFontClicked;
+
+			decrease_font = new Gtk.MenuItem (Catalog.GetString ("Decrease Font Size"));
+			decrease_font.AddAccelerator ("activate",
+						accel_group,
+						(uint) Gdk.Key.minus,
+						Gdk.ModifierType.ControlMask,
+						Gtk.AccelFlags.Visible);
+			decrease_font.Activated += DecreaseFontClicked;
+
 			Gtk.SeparatorMenuItem spacer2 = new Gtk.SeparatorMenuItem ();
 
 			bullets = new Gtk.CheckMenuItem (Catalog.GetString ("Bullets"));
 			bullets.Activated += ToggleBulletsClicked;
 
 			increase_indent = new Gtk.ImageMenuItem (Gtk.Stock.Indent, accel_group);
+			increase_indent.AddAccelerator ("activate",
+						accel_group,
+						(uint) Gdk.Key.Right,
+						Gdk.ModifierType.Mod1Mask,
+						Gtk.AccelFlags.Visible);
 			increase_indent.Activated += IncreaseIndentClicked;
 			increase_indent.Show ();
 
 			decrease_indent = new Gtk.ImageMenuItem (Gtk.Stock.Unindent, accel_group);
+			decrease_indent.AddAccelerator ("activate",
+						accel_group,
+						(uint) Gdk.Key.Left,
+						Gdk.ModifierType.Mod1Mask,
+						Gtk.AccelFlags.Visible);
 			decrease_indent.Activated += DecreaseIndentClicked;
 			decrease_indent.Show ();
 
@@ -1349,11 +1390,19 @@ namespace Tomboy
 			Append (normal);
 			Append (large);
 			Append (huge);
+			Append (increase_font);
+			Append (decrease_font);
 			Append (spacer2);
 			Append (bullets);
 			Append (increase_indent);
 			Append (decrease_indent);
 			ShowAll ();
+
+			theme_hack_menu = new Menu ();
+			theme_hack_menu.Realize ();
+			theme_hack_menu.StyleSet += delegate {
+				ModifyBg (StateType.Normal, theme_hack_menu.Style.Background (StateType.Normal));
+			};
 		}
 
 		protected override void OnShown ()
@@ -1466,6 +1515,42 @@ namespace Tomboy
 				buffer.SetActiveTag (tag);
 		}
 
+		void IncreaseFontClicked (object sender, EventArgs args)
+		{
+			if (event_freeze)
+				return;
+
+			if (buffer.IsActiveTag ("size:small")) {
+				buffer.RemoveActiveTag ("size:small");
+			} else if (buffer.IsActiveTag ("size:large")) {
+				buffer.RemoveActiveTag ("size:large");
+				buffer.SetActiveTag ("size:huge");
+			} else if (buffer.IsActiveTag ("size:huge")) {
+				// Maximum font size, do nothing
+			} else {
+				// Current font size is normal
+				buffer.SetActiveTag ("size:large");
+			}
+		}
+
+		void DecreaseFontClicked (object sender, EventArgs args)
+		{
+			if (event_freeze)
+				return;
+
+			if (buffer.IsActiveTag ("size:small")) {
+				// Minimum font size, do nothing
+			} else if (buffer.IsActiveTag ("size:large")) {
+				buffer.RemoveActiveTag ("size:large");
+			} else if (buffer.IsActiveTag ("size:huge")) {
+				buffer.RemoveActiveTag ("size:huge");
+				buffer.SetActiveTag ("size:large");
+			} else {
+				// Current font size is normal
+				buffer.SetActiveTag ("size:small");
+			}
+		}
+
 		void UndoClicked (object sender, EventArgs args)
 		{
 			if (undo_manager.CanUndo) {
