@@ -16,6 +16,7 @@ namespace Tomboy
 {
 	public class Tomboy : Application
 	{
+		static bool debugging;
 		static NoteManager manager;
 		static TomboyTrayIcon tray_icon;
 		static TomboyTray tray = null;
@@ -30,10 +31,42 @@ namespace Tomboy
 
 		public static void Main (string [] args)
 		{
+			// TODO: Extract to a PreInit in Application, or something
+#if WIN32
+			string tomboy_path =
+				Environment.GetEnvironmentVariable ("TOMBOY_PATH_PREFIX");
+			string tomboy_gtk_basepath =
+				Environment.GetEnvironmentVariable ("TOMBOY_GTK_BASEPATH");
+			Environment.SetEnvironmentVariable ("GTK_BASEPATH",
+				tomboy_gtk_basepath ?? string.Empty);
+			if (string.IsNullOrEmpty (tomboy_path)) {
+				string gtk_lib_path = null;
+				try {
+					gtk_lib_path = (string)
+						Microsoft.Win32.Registry.GetValue (@"HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\.NETFramework\AssemblyFolders\GtkSharp",
+						                                   string.Empty,
+						                                   string.Empty);
+				} catch (Exception e) {
+					Console.WriteLine ("Exception while trying to get GTK# install path: " +
+					                   e.ToString ());
+				}
+				if (!string.IsNullOrEmpty (gtk_lib_path))
+					tomboy_path =
+						gtk_lib_path.Replace ("lib\\gtk-sharp-2.0", "bin");
+			}
+			if (!string.IsNullOrEmpty (tomboy_path))
+				Environment.SetEnvironmentVariable ("PATH",
+				                                    tomboy_path +
+				                                    Path.PathSeparator +
+				                                    Environment.GetEnvironmentVariable ("PATH"));
+#endif
 			// Initialize GETTEXT
 			Catalog.Init ("tomboy", Defines.GNOME_LOCALE_DIR);
 
 			TomboyCommandLine cmd_line = new TomboyCommandLine (args);
+			debugging = cmd_line.Debug;
+			Logger.LogLevel = debugging ? Level.DEBUG : Level.INFO;
+			is_panel_applet = cmd_line.UsePanelApplet;
 
 #if ENABLE_DBUS || WIN32 || MAC // Run command-line earlier with DBus enabled
 			if (cmd_line.NeedsExecute) {
@@ -43,6 +76,8 @@ namespace Tomboy
 			}
 #endif // ENABLE_DBUS || WIN32
 
+			// NOTE: It is important not to use the Preferences
+			//       class before this call.
 			Initialize ("tomboy", "tomboy", "tomboy", args);
 
 			// Add private icon dir to search path
@@ -75,15 +110,16 @@ namespace Tomboy
 			}
 #endif
 
-			if (cmd_line.UsePanelApplet) {
+			if (is_panel_applet) {
 				tray_icon_showing = true;
-				is_panel_applet = true;
 
 				// Show the Close item and hide the Quit item
 				am ["CloseWindowAction"].Visible = true;
 				am ["QuitTomboyAction"].Visible = false;
 
 				RegisterPanelAppletFactory ();
+				Logger.Log ("All done.  Ciao!");
+				Exit (0);
 			} else {
 				RegisterSessionManagerRestart (
 				        Environment.GetEnvironmentVariable ("TOMBOY_WRAPPER_PATH"),
@@ -95,6 +131,11 @@ namespace Tomboy
 			Logger.Log ("All done.  Ciao!");
 		}
 
+		public static bool Debugging
+		{
+			get { return debugging; }
+		}
+
 		static string GetNotePath (string override_path)
 		{
 			// Default note location, as specified in --note-path or $TOMBOY_PATH
@@ -276,7 +317,8 @@ namespace Tomboy
 				"Sandy Armstrong <sanfordarmstrong@gmail.com>",
 				"Sebastian Rittau <srittau@jroger.in-berlin.de>",
 				"Kevin Kubasik <kevin@kubasik.net>",
-				"Stefan Schweizer <steve.schweizer@gmail.com>"
+				"Stefan Schweizer <steve.schweizer@gmail.com>",
+				"Benjamin Podszun <benjamin.podszun@gmail.com>"
 			};
 
 			string [] documenters = new string [] {
@@ -296,11 +338,16 @@ namespace Tomboy
 			about.Version = Defines.VERSION;
 			about.Logo = GuiUtils.GetIcon ("tomboy", 48);
 			about.Copyright =
-			        Catalog.GetString ("Copyright \xa9 2004-2007 Alex Graveley");
+			        Catalog.GetString ("Copyright \xa9 2004-2007 Alex Graveley\n" +
+				                   "Copyright \xa9 2004-2009 Others\n");
 			about.Comments = Catalog.GetString ("A simple and easy to use desktop " +
 			                                    "note-taking application.");
 			Gtk.AboutDialog.SetUrlHook (delegate (Gtk.AboutDialog dialog, string link) {
-				Services.NativeApplication.OpenUrl (link);
+				try {
+					Services.NativeApplication.OpenUrl (link);
+				} catch (Exception e) {
+					GuiUtils.ShowOpeningLocationError (dialog, link, e.Message);
+				}
 			}); 
 			about.Website = Defines.TOMBOY_WEBSITE;
 			about.WebsiteLabel = Catalog.GetString("Homepage");
@@ -357,6 +404,7 @@ namespace Tomboy
 
 	public class TomboyCommandLine
 	{
+		bool debug;
 		bool new_note;
 		bool panel_applet;
 		string new_note_name;
@@ -375,6 +423,12 @@ namespace Tomboy
 			Parse (args);
 		}
 
+		// TODO: Document this option
+		public bool Debug
+		{
+			get { return debug; }
+		}
+
 		public bool UsePanelApplet
 		{
 			get {
@@ -467,6 +521,9 @@ namespace Tomboy
 				bool quit = false;
 
 				switch (args [idx]) {
+				case "--debug":
+					debug = true;
+					break;
 #if ENABLE_DBUS || WIN32 || MAC
 				case "--new-note":
 					// Get optional name for new note...
